/*
 * Battery Module : Sample battery and Vref inputs and compute the current battery voltage.
 * compute battery low and exhaused conditions.
 * 
 */

//
// Analog Pins
//
#define VBATT_IN    0
#define VREF_IN     1

//
// Battery voltage thresholds
//
#define BATTERY_LOW 3.4
#define BATTERY_EXHAUSTED 3.0 

//
// Number of samples to include in our average (should be even value for most accurate computation)
//
#define NUM_BATT_SAMPLES 8

//
// Battery character glyph indicies in 5x7 font
//
#define BATT_FULL_BOX       0x7B
#define BATT_EMPTY_BOX      0x7C
#define BATT_EMPTY_L_ENDCAP 0x7D
#define BATT_EMPTY_R_ENDCAP 0x7E

//
// Variables
//
uint16_t _battVrefSamples[NUM_BATT_SAMPLES];
uint16_t _battVbattSamples[NUM_BATT_SAMPLES];
uint8_t _battPushIndex;
float _battCurVoltage;
boolean _battDisplayLowMessage;
char _battDispString[16];

//
// API routines
//
void InitBattery() {
  int i;
  uint16_t initVrefAdcVal;
  uint16_t initVbattAdcVal;
  
  // Get an initial set of values
  initVrefAdcVal = _GetAdcInput(VREF_IN);
  initVbattAdcVal = _GetAdcInput(VBATT_IN);
  
  // Initialize our variables with the initial sample
  for (i=0; i<NUM_BATT_SAMPLES; i++) {
    _battVrefSamples[i] = initVrefAdcVal;
    _battVbattSamples[i] = initVbattAdcVal;
  }
  _battPushIndex = 0;
  _battDisplayLowMessage = false;

  // Compute an initial voltage and set the display
  _ComputeBatteryVoltage();
  _UpdateBatteryDisplay();
}


void EvalBattery() {
  uint16_t curVrefAdcVal;
  uint16_t curVbattAdcVal;
  
  // Evaluate the battery state once per second
  if (timerSecTick) {
    // Get a set of values
    curVrefAdcVal = _GetAdcInput(VREF_IN);
    curVbattAdcVal = _GetAdcInput(VBATT_IN);
    
    // Push the values into our averaging array
    _battVrefSamples[_battPushIndex] = curVrefAdcVal;
    _battVbattSamples[_battPushIndex++] = curVbattAdcVal;
    if (_battPushIndex >= NUM_BATT_SAMPLES) _battPushIndex = 0;
    
    // Compute the current battery voltage and set the display
    _ComputeBatteryVoltage();
    if (EnableDisplayUpdates()) {
      _UpdateBatteryDisplay();
    }
  }
}


boolean BatteryIsExhausted() {
  // Check against 0.1 volts is to allow us to power the device through the programming interface 
  return ((_battCurVoltage >= 0.1) && (_battCurVoltage <= BATTERY_EXHAUSTED));
}


//
// Internal routines
//
uint16_t _GetAdcInput(int n) {
  // This routine takes 6 readings, throws out the highest and lowest values and then
  // returns a rounded average of the remaining 4 readings.
  uint16_t curAdc;
  uint16_t minAdc = 65535;
  uint16_t maxAdc = 0;
  uint16_t sum;
  int i;
  
  // Get readings
  sum = 0;
  for (i=0; i<6; i++) {
    curAdc = analogRead(n);
    sum += curAdc;
    if (curAdc < minAdc) minAdc = curAdc;
    if (curAdc > maxAdc) maxAdc = curAdc;
  }
  
  // Throw away high and low readings
  sum = sum - (minAdc + maxAdc);
  
  // Compute average by using sum[17:2] and rounding on sum[1]
  curAdc = sum >> 2;
  if ((curAdc < 1023) && (sum & 0x2)) {
    // Round up if necessary
    curAdc++;
  }
  
  return curAdc;  
}


uint16_t _ComputeAverageBattCount(uint16_t *smplArray) {
  uint16_t i;
  uint16_t sum;
  
  sum = 0;
  for (i=0; i<NUM_BATT_SAMPLES; i++) {
    sum += *(smplArray+i);
  }
  
  i = sum / NUM_BATT_SAMPLES;
  if ((i < 65535) && ((sum % NUM_BATT_SAMPLES) >= (NUM_BATT_SAMPLES/2))) {
    // Round up
    i++;
  }
  return i;
}


void _ComputeBatteryVoltage() {
  uint16_t VrefAvg;
  uint16_t VbattAvg;
  
  // Get the average count values
  VrefAvg = _ComputeAverageBattCount(_battVrefSamples);
  VbattAvg = _ComputeAverageBattCount(_battVbattSamples);
  
  // VBATT_IN = Battery Voltage / 2 (to make it fit in the 2.5 volt reference range)
  // Battery voltage = (VbattVal / VrefVal) * 2.5 * 2
  _battCurVoltage = ((float) VbattAvg / (float) VrefAvg) * 5;
}


// Returns a string with 2 fractional digits
char *_GetBattVoltageString()
{
  // Add 0.005 to voltage to force rounding with 2 fractional digits
 char *a = _battDispString;
 long intPortion = (long) (_battCurVoltage + 0.005);
 long frac = abs((long)(((_battCurVoltage + 0.005) - intPortion) * 100));
 
 itoa(intPortion, a, 10);
 while (*a != '\0') a++;
 *a++ = '.';
 if (frac < 10) {
   // Add a leading zero
   *a++ = '0';
 }
 itoa(frac, a, 10);
 return _battDispString;
}


void _UpdateBatteryDisplay() {
  // Battery display is on the first (lowest) small font line of the display.  It consists of two parts:
  //   1. Left justified bargraph made of box characters with each character representing 0.1 volts (3 - 4.1 volts)
  //   2. Right justified battery voltage "X.XXv" or "Low" alternating indicator
  int i;
  float t;
  boolean isLow = _battCurVoltage <= BATTERY_LOW;
  
  // Create and display the bargraph
  t = 3.0;
  for (i=0; i<12; i++) {
    if (_battCurVoltage >= t) {
      _battDispString[i] = BATT_FULL_BOX;
    } else {
      if (i == 0) {
        _battDispString[i] = BATT_EMPTY_L_ENDCAP;
      } else if (i == 11) {
        _battDispString[i] = BATT_EMPTY_R_ENDCAP;
      } else {
        _battDispString[i] = BATT_EMPTY_BOX;
      }
    }
    t += 0.1;
  }
  _battDispString[i] = 0;      // Terminator
  OLED_PrintfSetColumnRow(0, 0, SMALL_FONT);
  OLED_Printf(_battDispString);
  
  // Create and display the voltage or low message
  if (isLow) {
    _battDisplayLowMessage = !_battDisplayLowMessage;
  } else {
    _battDisplayLowMessage = false;
  }
  OLED_PrintfSetColumnRow(90, 0, SMALL_FONT);
  if (_battDisplayLowMessage) {
    OLED_Printf("  Low ");
  } else {
    OLED_Printf("%s v", _GetBattVoltageString());
  }
}
