/*
 * Misc Functions Module
 *
 * short button press turns on, resets auto-off timer
 * long button press turns off
 *
 * Blinks Status LED during final minute of auto-off timer sequence.
 */

//
// Digital IO Pins
//
#define STATUS_LED  8
#define HOLD_OUT    A2

//
// Auto power-off timeout (seconds)
//
#define AUTO_OFF_TO 300

//
// Internal variables
//
uint16_t _autoOffTimer;
boolean _powerEnabled;

//
// API routines
//
void InitMisc() {
  // Initialize our IO pins
  pinMode(STATUS_LED, OUTPUT);
  pinMode(HOLD_OUT, OUTPUT);
  
  // Initialize our auto-off timer
  _autoOffTimer = AUTO_OFF_TO;
  
  // Initial state
  _EnablePower(true);
  _SetStatusLed(false);
}


void EvalMisc() {
  // Evaluate our auto-off timer
  if (timerSecTick) {
    --_autoOffTimer;
    if (EnableDisplayUpdates()) {
      _UpdateAutoOffDisplay();
    }
    _UpdateStatusLed();
  }
  
  // Look for power off triggers
  if (ButtonLongPress() || BatteryIsExhausted() || (_autoOffTimer == 0)) {
    // Clear the display.  We do this so the OLED RAM is clear to prevent a stale display on the next power-up.
    // It appears that the leakage current through our power-control MOSFET is enough to keep the SRAM contents valid.
    Fill_RAM(0x00);
    
    // Then power down
    _EnablePower(false);
  }
  
  // Look for a reset of the auto-off timer
  if (ButtonShortPress()) {
    _autoOffTimer = AUTO_OFF_TO;
  }
}


boolean EnableDisplayUpdates() {
  return _powerEnabled;
}


//
// Internal functions
//
void _EnablePower(boolean en) {
  if (en) {
    digitalWrite(HOLD_OUT, HIGH);
  } else {
    digitalWrite(HOLD_OUT, LOW);
  }
  _powerEnabled = en;
}


void _UpdateAutoOffDisplay() {
  OLED_PrintfSetColumnRow(109, 1, SMALL_FONT);
  if (_autoOffTimer < 10) {
    OLED_Printf("  %d", _autoOffTimer);
  } else if (_autoOffTimer <= 60) {
    OLED_Printf(" %d", _autoOffTimer);
  } else {
    OLED_Printf("   ");
  }      
}


void _UpdateStatusLed() {
  if (_autoOffTimer <= 60) {
    // Blink
    _SetStatusLed((_autoOffTimer & 0x01) == 0x01);
  } else {
    _SetStatusLed(false);
  }
}


void _SetStatusLed(boolean en) {
  if (en) {
    digitalWrite(STATUS_LED, LOW);
  } else {
    digitalWrite(STATUS_LED, HIGH);
  }
}

