/*
 * Read 10x/second and compute average temperature
 */
#include <Wire.h>


//
// MLX90614 I2C address
//
#define I2C_ADDR    0x5A

//
// MLX90614 RAM address offsets
//
#define TA_ADDR     0x06
#define TO_ADDR     0x07

//
// Number of eval periods between samples
//
#define TEMP_EVAL_PER_SAMPLE 4
//
// Number of samples in our average (should be an even value to make the computation most accurate)
//
#define TEMP_NUM_SAMPLES 4

uint16_t _tempSamples[TEMP_NUM_SAMPLES];
uint8_t _tempPushIndex;
uint8_t _tempEvalCounter;
float _tempCurValC;
char _tempString[5];


void InitTemp() {
  uint16_t curTempCount;
  int i;
  
  // Initialize the interfaces
  Wire.begin();
  Serial.begin(115200);
  
  // Get an initial reading
  curTempCount = GetIrSensorVal();
  
  // Initialize our variables with the initial sample
  _tempEvalCounter = TEMP_EVAL_PER_SAMPLE;
  for (i=0; i<TEMP_NUM_SAMPLES; i++) {
    _tempSamples[i] = curTempCount;
  }
  _tempPushIndex = 0;
  
  // Compute an average and set the initial display
  _ComputeAverageTemp();
  _UpdateTempDisplay();
}


void EvalTemp() {
  if (timerTick) {
    if (--_tempEvalCounter == 0) {
      // Reset eval counter
      _tempEvalCounter = TEMP_EVAL_PER_SAMPLE;
      
      // Get a new value and push it into our averaging array
      _tempSamples[_tempPushIndex++] =  GetIrSensorVal();
      if (_tempPushIndex >= TEMP_NUM_SAMPLES) _tempPushIndex = 0;
      
      // Compute an average and update the display
      _ComputeAverageTemp();
      if (EnableDisplayUpdates()) {
        _UpdateTempDisplay();
      }
      _OutputSerialTemp();
    }
  }
}


//
// Internal routines
//
uint16_t GetIrSensorVal() {
  uint16_t v;
  uint8_t pec;
  
  // Load the address
  Wire.beginTransmission(I2C_ADDR);
  Wire.write(TO_ADDR);
  Wire.endTransmission(false);
  
  // Get the RAM location and PEC value (which we throw away)
  if (Wire.requestFrom(I2C_ADDR, 3) == 3) {
    v = Wire.read();
    v |= (Wire.read() << 8);
    pec = Wire.read();
    if (v & 0x8000) {
      return 0;
    } else {
      return v;
    }
  } else {
    return 0;
  }
}


void _ComputeAverageTemp() {
  uint16_t i;
  uint32_t sum;
  
  // Get an average count value
  sum = 0;
  for (i=0; i< TEMP_NUM_SAMPLES; i++) {
    sum += _tempSamples[i];
  }
  i = sum / TEMP_NUM_SAMPLES;
  if ((i < 65535) && ((sum % TEMP_NUM_SAMPLES) >= (TEMP_NUM_SAMPLES/2))) {
    // Round up
    i++;
  }
  
  // Convert to degrees C
  //  Note: Temp in degrees K = Count * 0.02
  //        So Temp in C = Temp in K - 273.15
  _tempCurValC = (float) i * 0.02 - 273.15;
}


// Returns a 5-character string with one fractional digit and zero, one or two leading spaces
char *_GetTempString(float t)
{
 char *a = _tempString;
 t = t + 0.05;  // Will force rounding for one fractional digit
 long intPortion = (long) t;
 long frac = abs((long)((t - intPortion) * 10));

 // Leading 0 blanking (spaces)
 if (intPortion < 0) {
   if (intPortion > -10) {
     *a++ = ' ';
   }
 } else {
   if (intPortion < 100) {
     if (intPortion < 10) {
       *a++ = ' ';
     }
     *a++ = ' ';
   }
 }
 
 // Convert the number itself
 itoa(intPortion, a, 10);
 while (*a != '\0') a++;
 *a++ = '.';
 itoa(frac, a, 10);
 return _tempString;
}


void _UpdateTempDisplay() {
  //
  // Display temp C on top line, temp F on next line
  //    Format XXX.X C
  //           XXX.X F
  //
  float tempF = _tempCurValC * 1.8 + 32;
  
  // Temp C
  OLED_PrintfSetColumnRow(10, 7, LARGE_FONT);
  OLED_Printf("%s C", _GetTempString(_tempCurValC));
  
  // Temp F
  OLED_PrintfSetColumnRow(10, 4, LARGE_FONT);
  OLED_Printf("%s F", _GetTempString(tempF));
}


void _OutputSerialTemp() {
  Serial.println(_tempCurValC);
}
